jQuery.plugin("de_epages.productcomparison.inc.productcompare", {
    plugin: ['ep.core', 'ep.ui.tooltip', 'de_epages.core', 'ep.ui.Input', 'jQuery.storage', 'jQuery.ui.widget']
}, function ($, undefined) {

    $.widget('de_epages.productcomparison.inc.productCompare', {

        options: {
            maxItemNo: 8,
            minItemNo: 2,
            productID: undefined,
            tooltipTooManyProducts: undefined,
            tooltipNoProducts: undefined
        },

        _create: function () {
            var self = this.element,
                checkboxID = self.attr('id'),
                productID = this.options.productID,
                label = $('#' + checkboxID).nextAll('label'),
                link = label.children('a').first();

            /* check product id */
            if (!this._check(productID, link)) {
                return;
            }

            this.options.label = label;
            this.options.checkboxID = checkboxID;

            /* add 'click' event handler on associated 'Compare' link */
            label.on('click', $.proxy(function (event) {
                this._handleClick(event, link, self);
            }, this));

            /* create checkbox, (un)tick checkbox, and add 'change' event handler */
            ep(self).uiInput();
            this._tick(productID);
            self.on('change', $.proxy(function (event) {
                this._handleChange(event, self);
            }, this));
        },

        _check: function (productID, link) {
            if (productID === undefined || isNaN(parseInt(productID)) || link.length === 0) {
                console.warn('ProductComparison: parameter \'productID\' and/or \'compare\' link missing; product comparison is removed from SF.');
                self.remove();
                if (label.length > 0) {
                    label.remove();
                }
                return false;
            }
            return true;
        },

        _tick: function (productID) {
            var productList = $.sessionStorage('Product::ProductComparison') || [],
                isSelected = ($.inArray(productID, productList) > -1) ? true : false;
            this.element.prop('checked', isSelected);
        },

        _handleClick: function (event, target, parent) {
            event.preventDefault();
            var productIDs = $.sessionStorage('Product::ProductComparison') || [],
                url = target.attr('href');

            this._addProductToComparisonList(this.options.productID, productIDs);
            $.sessionStorage('Product::ProductComparison', productIDs);
            if (productIDs.length < this.options.minItemNo) {
                parent.prop('checked', true);
                this._showTooltip('NoProductsInComparisonList', this.options.tooltipNoProducts, target);
                return;
            }

            $.each(productIDs, function (index, value) {
                url += '&SelectedObjectID=' + value;
            });
            window.location = url;
        },

        _handleChange: function (event, target) {
            var productIDs = $.sessionStorage('Product::ProductComparison') || [],
                productID = this.options.productID,
                isSelected = false;

            target = $('#' + target.attr('id'));
            if (target.is(':checked')) { // add product to comparison list
                if (productIDs.length >= this.options.maxItemNo) {
                    target.prop('checked', isSelected);
                    this._showTooltip('TooManyComparisonProducts', this.options.tooltipTooManyProducts, target);
                    return;
                }
                this._addProductToComparisonList(productID, productIDs);
                isSelected = true;
            } else { // remove product from comparison list
                productIDs = $.grep(productIDs, function (value) {
                    return value != productID;
                });
            }
            target.prop('checked', isSelected);
            $.sessionStorage('Product::ProductComparison', productIDs);
        },

        _addProductToComparisonList: function (productID, productIDs) {
            var index = $.inArray(productID, productIDs);
            if (index > -1) { // deletes former occurrence
                productIDs.splice(index, 1);
            }
            productIDs.push(productID);
            while (productIDs.length > this.options.maxItemNo) {
                productIDs.shift();
            }
        },

        _showTooltip: function (id, message, element) {
            if (message === undefined) {
                console.warn('ProductComparison: No tooltip message is defined.');
                return;
            }

            var template = ep($('#' + id + 'Tooltip'));
            if (!template[0]) {
                template = ep('<div id="' + id + 'Tooltip" class="Message Medium Notification Wide"><div class="Headline"></div><div class="MessageDetails"><span class="ExtractedContentAreaElement">' + message + '</span></div></div>').uiTooltip({
                    interactive: true,
                    orientation: 'bottom center',
                    event: 'click'
                });
            }
            template.uiTooltip({
                context: element
            });
            template.uiTooltip('show');
        },

        destroy: function () {
            var options = this.options;
            ep(this.element).uiInput('destroy');
            $('#' + options.checkboxID).off('change');
            options.label.off('click');

            return this.Inherited(arguments);
        }

    });

});
