use 5.010;
use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::File qw ( GetFileContent );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    JoinObjectPath
    LoadObjectByPath
    LoadRootObject
);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::UniversalPortal::API::CSV::UniversalProductsExportDriver;
use Readonly;
use Test::More;
use File::Temp qw( tempfile tempdir );

Readonly my $ROOTPATH => '/TestUniversalPortal';

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            # startUp
            ok( Init(), 'initialization' );
            # tests
            eval {
                TestCSVExport();
            };

            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage );
            }

            # shutDown
            ok( Cleanup(), 'cleanup' );
            done_testing();
        }
    );
}

sub TestCSVExport {
    ok (my $ExporterObject = DE_EPAGES::UniversalPortal::API::CSV::UniversalProductsExportDriver->new(),'Load ExporterObject');
    ok (my $PortalConfig = LoadObjectByPath($ROOTPATH)->path( JoinObjectPath( [ 'TestShopUniversal', 'PortalConfigs', 'UniversalPortal' ] ) ), 'Load portal config');
    isa_ok( $PortalConfig, 'DE_EPAGES::UniversalPortal::API::Object::UniversalPortalConfig' );
    my $dir = tempdir( CLEANUP => 1 );
    my (undef, $FileName) = tempfile( DIR => $dir );
    note 'Tempfile: '.$FileName."\n";
    ok(my $hFileFormat = {
        encoding        => 'utf8',
        sep_char        => '|',
        eol             => "\n",
        quote_char      => '"',
        always_quote    => 1,
    },'create csv formatters');
    my @ColumnNames = [];
    foreach my $UniversalPortalAttributeSet (@{$PortalConfig->get('UniversalPortalAttributeSets')}) {
            push(@ColumnNames, $UniversalPortalAttributeSet->alias);
    }
    $ExporterObject->exportCSV( $FileName, $hFileFormat, \@ColumnNames, $PortalConfig, [] );
    ok (-f $FileName,'export file created');
    is(${GetFileContent($FileName)},'','testCSVExporterWithNoAttributes.t - is result of unclompletd Portal - empty string');
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'TestUniversalPortal' } );
    ImportObjects( 'import_without_attributes.xml', $ROOTPATH );
    return 1;
}

RunScript( Sub => \&Main );

1;
