#!/bin/env perl

use strict;
use File::Basename qw(dirname);
use feature qw(say);
use lib qw(. Install);
use Hotfix qw(GetHostName);

use DE_EPAGES::Core::API::File qw ( CopyDirectory GetFileContentLines MoveFile );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::SystemCommand qw (SystemCommand);

sub Main {
    local $| = 1;

    chdir dirname $0;

    my $BackupDir = '../Backup-'. GetHostName();

    if (-f '../delete-files.txt') {
        foreach (@{GetFileContentLines('../delete-files.txt')}) {
            my ($Dir, $File) = split;
            next unless -e "$ENV{$Dir}/$File";
            say qq(DeleteFile($ENV{$Dir}/$File));
            MoveFile("$ENV{$Dir}/$File", "$BackupDir$ENV{$Dir}/$File",
                     { create_directory => 1 } );

        }
    }

    return unless -d '../FileChanges';
    chdir '../FileChanges';

    SystemCommand( '/etc/init.d/epages6 perm all -d srv/epages/eproot', { Print => 1 })
        unless $^O eq 'MSWin32';

    foreach my $Dir (qw(srv/epages/etc srv/epages/eproot/bin
                        srv/epages/eproot/Setup srv/epages/eproot/WebAdapter)) {
        next if $^O eq 'MSWin32' and $Dir ne 'srv/epages/eproot/bin';
        next unless -d $Dir;
        say qq(CopyDirectory($Dir, "/$Dir", "$BackupDir/$Dir"));
        CopyDirectory($Dir, "/$Dir", "$BackupDir/$Dir", {by => 'perl', stat => 'preserve'});
    }

    return if $^O eq 'MSWin32';

    foreach my $Dir (sort glob '*') {
        next if $Dir eq 'srv';
        say qq(CopyDirectory($Dir, "/$Dir", "$BackupDir/$Dir"));
        CopyDirectory($Dir, "/$Dir", "$BackupDir/$Dir", {by => 'perl', stat => 'preserve'});
    }
    SystemCommand( '/etc/init.d/epages6 perm admin', { Print => 1 });
}

RunScript(
    'Sub' => \&Main
);
