#!/bin/env perl

use strict;
use File::Basename qw ( dirname );
use File::Find qw ( find );
use feature qw( say );

use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Config qw( StoreType GetBusinessUnits StoreTypesWithCartridge );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::Object::API::XML::ObjectDeleteHandler qw (DeleteObjects);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw( ImportObjects );
use DE_EPAGES::Trigger::API::XML::HookImportHandler qw (ImportHooks);

sub Main {
    local $| = 1;

    chdir dirname $0;

    my $SourceDir = '35/XML';
    return unless -d $SourceDir;
    chdir $SourceDir;

    my $XMLFiles = _GetImports();

    foreach my $Store (keys %$XMLFiles) {
        RunOnStore(
            Store => $Store,
            Sub => sub  {
                foreach my $XML (sort _ByName @{$XMLFiles->{$Store}}) {
                    if ($XML->{file} =~ m,/([0-9]+-)?Hooks[^/]+$,) {
                        say qq(RunOnStore $Store: ImportHooks($XML->{file}));
                        ImportHooks($XML->{file});
                    } elsif ($XML->{file} =~ m,/([0-9]+-)?imp_[^/]+$,) {
                        say qq(RunOnStore $Store: ImportObjects($XML->{file}, $XML->{path}));
                        ImportObjects($XML->{file}, $XML->{path});
                    } elsif ($XML->{file} =~ m,/([0-9]+-)?del_[^/]+$,) {
                        say qq(RunOnStore $Store: DeleteObjects($XML->{file}, $XML->{path}));
                        DeleteObjects($XML->{file}, $XML->{path});
                    }
                }
            },
            );
    }
}

sub _ByName($$) {
    my ($a, $b) = @_;
    my $aFile = $a->{file};
    $aFile =~ s,.*/,,;
    my $bFile = $b->{file};
    $bFile =~ s,.*/,,;

    return $aFile cmp $bFile;
}

sub _GetImports {
    my ($Prefix) = @_;
    my $BU;
    push @{$BU->{StoreType($_)}}, $_ foreach @{GetBusinessUnits({conf => 1})};

    my $XMLFiles = {};
    find(
        sub {
            return unless /\.xml$/;
            return if $Prefix and $_ !~ /^([0-9]+-)?$Prefix.*\.xml$/;
            my $Cartridge = (split /[\\\/]/, $File::Find::name)[1];
            my @Dir = split(/[\\\/]/, $File::Find::dir);
            my $Path = join('/', @Dir[2..$#Dir]);
            $Path = undef unless $Path;
            push @{$XMLFiles->{$_}}, {file => $File::Find::name, path => $Path}
            foreach @{_StoresWithCartridge($Cartridge, $BU)};
        }, '.'
        );
    return $XMLFiles;
}

sub _StoresWithCartridge ($$) {
    my ($Cartridge, $BU) = @_;

    my @Stores;
    foreach my $StoreType (@{StoreTypesWithCartridge("DE_EPAGES::$Cartridge")}) {
        push @Stores, $_ foreach @{$BU->{$StoreType}};
    }
    return \@Stores;
}

RunScript(
    'Sub' => \&Main
);
