#!/bin/env perl

use strict;
use File::Basename;
use feature qw(say);

use DE_EPAGES::Core::API::File qw( GetFileContent DeleteFile DeleteDirectory );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Config qw( BUStoreTypes );
use DE_EPAGES::Installer::API::Install qw( InstallStoreTypeDir );
use DE_EPAGES::JQuery::API::DictionaryConverter qw ( ConvertCartridgeDictionary ConvertSubDictionary );
use DE_EPAGES::Dojo::API::Shrinksafe qw ( RunDojoShrinksafe );
use DE_EPAGES::Presentation::API::CSSPackageBuilder qw ( RunCSSPackaging );
use DE_EPAGES::Presentation::API::JSPackageBuilder qw ( RunJavaScriptPackaging );
use DE_EPAGES::XML::API::XML::CartridgeInstallerImportHandler qw( GetCartridges );

sub Main {
    local $| = 1;

    chdir dirname $0;
    chdir '..';

    my $Version = GetFileContent('version.txt');
    chomp $$Version;
    $$Version =~ s/^(([0-9]+\.){2}[0-9]+).*/$1/;

    my @StoreTypes = @{BUStoreTypes()};

    my $haCartridges = GetCartridges();

    my @CustomStoreTypes = split(/\s+/, $ENV{EPAGES_CUSTOM_STORETYPES});
    push @CustomStoreTypes, 'Master' unless 'Master' ~~ @CustomStoreTypes;
    push @CustomStoreTypes, 'MultiStore' unless 'MultiStore' ~~ @CustomStoreTypes;

    foreach (@CustomStoreTypes) {
        next unless $_ ~~ @StoreTypes;

        # cleanfiles_storetype
        say qq(DeleteFile("$ENV{EPAGES_CONFIG}/StoreTypes.d/$$Version/$_.conf"));
        DeleteFile("$ENV{EPAGES_CONFIG}/StoreTypes.d/$$Version/$_.conf");
        say qq(DeleteDirectory("$ENV{EPAGES_WEBROOT}/StoreTypes/$$Version/$_"));
        DeleteDirectory("$ENV{EPAGES_WEBROOT}/StoreTypes/$$Version/$_", {recursive => 1});

        # Installer/Scripts/installStoreType.pl
        say qq(InstallStoreTypeDir($_, GetCartridges(), $$Version));
        InstallStoreTypeDir($_, $haCartridges->{$_}, $$Version);

        # JQuery/Scripts/DictionaryConverter.pl
        foreach my $Cartridge (@{$haCartridges->{$_}}){
            say qq(ConvertCartridgeDictionary({version => $$Version, storetype => $_, cartridge => $Cartridge}));
            ConvertCartridgeDictionary({version => $$Version, storetype => $_,
                                        cartridge => $Cartridge});
            say qq(ConvertSubDictionary({version => $$Version, storetype => $_, cartridge => $Cartridge}));
            ConvertSubDictionary({version => $$Version, storetype => $_,
                                  cartridge => $Cartridge});
        }

        # Dojo/Scripts/build.pl
        say qq(RunDojoShrinksafe({Version => $$Version, StoreType => $_}));
        RunDojoShrinksafe({Version => $$Version, StoreType => $_});

        # Presentation/Scripts/buildJS.pl
        say qq(RunJavaScriptPackaging({Version => $$Version, StoreType => $_}));
        RunJavaScriptPackaging({Version => $$Version, StoreType => $_});

        # Presentation/Scripts/buildCSS.pl
        say qq(RunCSSPackaging({Version => $$Version, StoreType => $_}));
        RunCSSPackaging({Version => $$Version, StoreType => $_});
    }
}

RunScript(
    'Sub' => \&Main
);
